from typing import Optional

from airflow_pydantic import CallablePath, ImportPath, PythonTask, PythonTaskArgs
from pydantic import Field

__all__ = (
    "SkipOperatorArgs",
    "SkipOperator",
    "FailOperatorArgs",
    "FailOperator",
    "PassOperatorArgs",
    "PassOperator",
)


class SkipTaskArgs(PythonTaskArgs):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.skip", description="python_callable")


# Alias
SkipOperatorArgs = SkipTaskArgs


class FailTaskArgs(PythonTaskArgs):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.fail", description="python_callable")


# Alias
FailOperatorArgs = FailTaskArgs


class PassTaskArgs(PythonTaskArgs):
    python_callable: Optional[CallablePath] = Field(default="airflow_common_operators.pass_", description="python_callable")


# Alias
PassOperatorArgs = PassTaskArgs


class SkipTask(PythonTask, SkipOperatorArgs):
    operator: ImportPath = Field(default="airflow_pydantic.airflow.PythonOperator", description="airflow operator path", validate_default=True)


# Alias
SkipOperator = SkipTask


class FailTask(PythonTask, FailOperatorArgs):
    operator: ImportPath = Field(default="airflow_pydantic.airflow.PythonOperator", description="airflow operator path", validate_default=True)


# Alias
FailOperator = FailTask


class PassTask(PythonTask, PassOperatorArgs):
    operator: ImportPath = Field(default="airflow_pydantic.airflow.PythonOperator", description="airflow operator path", validate_default=True)


# Alias
PassOperator = PassTask



from airflow_common_operators import GitRepo, PipLibrary, clone_repo, in_bash, in_conda, in_virtualenv


class TestLibrary:
    def test_pip(self):
        p = PipLibrary(name="tmp", version_constraint="<5", install_deps=False, tool="uv")
        assert p.install() == "uv pip install --no-deps tmp<5"
        p = PipLibrary(name="tmp", version_constraint="", install_deps=True, tool="pip")
        assert p.install() == "pip install tmp"

    def test_git(self):
        g = GitRepo(name="tmp", repo="tmp", branch="main", install=True, install_deps=False, tool="uv")
        assert (
            g.clone()
            == "\n[[ -d tmp ]] || git clone tmp\npushd tmp\ngit stash\ngit clean -fdx\ngit fetch --all --force\ngit checkout main\ngit reset origin/main --hard\n\nuv pip install --no-deps -e .\n"
        )
        g = GitRepo(name="tmp", repo="tmp", branch="main", install=True, install_deps=True, tool="pip")
        assert (
            g.clone()
            == "\n[[ -d tmp ]] || git clone tmp\npushd tmp\ngit stash\ngit clean -fdx\ngit fetch --all --force\ngit checkout main\ngit reset origin/main --hard\n\npip install -e .\n"
        )
        assert (
            clone_repo(name="tmp", repo="tmp", branch="main", install=False, install_deps=True, tool="uv")
            == "\n[[ -d tmp ]] || git clone tmp\npushd tmp\ngit stash\ngit clean -fdx\ngit fetch --all --force\ngit checkout main\ngit reset origin/main --hard\n"
        )

    def test_in_bash(self):
        assert in_bash('a simple "test"', escape=False, quote="'", login=True) == "bash -lc 'a simple \"test\"'"
        assert in_bash("a simple 'test'", escape=True, quote=False, login=False) == "bash -c 'a simple '\"'\"'test'\"'\"''"

    def test_in_conda(self):
        assert in_conda("a", "b") == "micromamba activate a && b"

    def test_in_virtualenv(self):
        assert in_virtualenv("a", "b") == "source a/bin/activate && b"



import pytest
from airflow_pydantic.airflow import AirflowFailException, AirflowSkipException

from airflow_common_operators import fail, pass_, skip


class TestCommon:
    def test_pass(self):
        pass_()

    def test_skip(self):
        with pytest.raises(AirflowSkipException):
            skip()

    def test_fail(self):
        with pytest.raises(AirflowFailException):
            fail()



import pytest

from airflow_common_operators import DagCleanupTask


class TestModels:
    def test_dag_cleanup(self):
        try:
            from airflow import DAG
        except ImportError:
            return pytest.skip("Airflow DAG not available in this environment")

        d = DAG(dag_id="test_dag_cleanup")
        t = DagCleanupTask(dag=d, task_id="test_dag_cleanup_task")
        assert t



from datetime import datetime, timedelta
from typing import TYPE_CHECKING, Optional, Type

from airflow_pydantic import CallablePath, Task, TaskArgs
from pydantic import BaseModel, Field, field_validator
from pytz import UTC

if TYPE_CHECKING:
    from airflow.operators.python import PythonOperator

__all__ = (
    "DagCleanupParams",
    "DagCleanupOperatorArgs",
    "DagCleanupTaskArgs",
    "DagCleanupOperator",
    "DagCleanupTask",
)


class DagCleanupParams(BaseModel):
    delete_successful: Optional[bool] = Field(default=True)
    delete_failed: Optional[bool] = Field(default=True)
    mark_failed_as_successful: Optional[bool] = Field(default=False)
    max_dagruns: Optional[int] = Field(default=10)
    days_to_keep: Optional[int] = Field(default=10)


class DagCleanupTaskArgs(DagCleanupParams, TaskArgs, extra="allow"): ...


# Alias
DagCleanupOperatorArgs = DagCleanupTaskArgs


def create_cleanup_dag_runs(**kwargs) -> "PythonOperator":
    from airflow.models import DagModel, DagRun
    from airflow.operators.python import PythonOperator
    from airflow.utils.session import provide_session
    from airflow.utils.state import State

    @provide_session
    def _cleanup_dag_runs(session=None, **context):
        params = context["params"]

        # Get the configurable parameters
        delete_successful = params.get("delete_successful", DagCleanupTaskArgs.model_fields["delete_successful"].default)
        delete_failed = params.get("delete_failed", DagCleanupTaskArgs.model_fields["delete_failed"].default)
        mark_failed_as_successful = params.get("mark_failed_as_successful", DagCleanupTaskArgs.model_fields["mark_failed_as_successful"].default)
        max_dagruns = params.get("max_dagruns", DagCleanupTaskArgs.model_fields["max_dagruns"].default)
        days_to_keep = params.get("days_to_keep", DagCleanupTaskArgs.model_fields["days_to_keep"].default)

        # Make cutoff_date timezone-aware (UTC)
        utc_now = datetime.utcnow().replace(tzinfo=UTC)
        cutoff_date = utc_now - timedelta(days=days_to_keep)

        # Fetch all DAGs from the DagBag
        dag_ids = [d.dag_id for d in session.query(DagModel.dag_id).distinct(DagModel.dag_id).all()]

        deleted = 0

        for dag_id in dag_ids:
            print(f"Cleaning up DAG: {dag_id}")

            # Query for DAG runs of each DAG
            query = session.query(DagRun).filter(DagRun.dag_id == dag_id)

            if delete_successful is False:
                query = query.filter(DagRun.state != State.SUCCESS)
            if delete_failed is False:
                query = query.filter(DagRun.state != State.FAILED)

            dagruns = query.order_by(DagRun.execution_date.asc()).all()
            total_runs = len(dagruns)

            for dr in dagruns:
                # Compare execution_date (offset-aware) with cutoff_date (now offset-aware)
                if dr.execution_date < cutoff_date or total_runs > max_dagruns:
                    session.delete(dr)
                    deleted += 1
                    total_runs -= 1  # Adjust count since we deleted one
                elif mark_failed_as_successful:
                    # Need to iterate through all remaining
                    if dr.state == State.FAILED:
                        # Mark failed runs as successful
                        dr.state = State.SUCCESS
                        session.merge(dr)
                elif not mark_failed_as_successful:
                    break  # Since they are ordered, no more to delete

        session.commit()
        print(f"Total DAG runs deleted: {deleted}")

    operator = PythonOperator(
        python_callable=_cleanup_dag_runs,
        **kwargs,
    )
    return operator


class DagCleanupTask(Task, DagCleanupTaskArgs):
    operator: CallablePath = Field(default="airflow_common_operators.tasks.dag_clean.create_cleanup_dag_runs", validate_default=True)

    @field_validator("operator")
    @classmethod
    def validate_operator(cls, v: Type) -> Type:
        if v.__qualname__ != create_cleanup_dag_runs.__qualname__:
            raise ValueError(f"operator must be 'airflow_common_operators.tasks.dag_clean.create_cleanup_dag_runs', got: {v}")
        return v


# Alias
DagCleanupOperator = DagCleanupTask



from unittest.mock import MagicMock

import pytest

from airflow_common_operators.models import DagCleanup


class TestModels:
    def test_dag_cleanup(self):
        try:
            from airflow.models import DagModel  # noqa: F401
        except ImportError:
            return pytest.skip("Airflow DagModel not available in this environment")

        d = DagCleanup()
        d.cleanup_dag_runs(session=MagicMock(), params={})



import pytest
from airflow_config import load_config


class TestConfig:
    def test_cleanup_task(self):
        conf = load_config("config", "config")

        try:
            from airflow_config import DAG
        except ImportError:
            return pytest.skip("Airflow DAG not available in this environment")

        d = DAG(dag_id="test_cleanup", config=conf)
        assert len(d.tasks) == 1

    def test_cleanup_render(self):
        conf = load_config("config", "config")
        assert (
            conf.dags["test_cleanup"].render()
            == """# Generated by airflow-config
from airflow.models import DAG
from airflow.models.param import Param
from airflow_common_operators.tasks.dag_clean import create_cleanup_dag_runs
from datetime import datetime

with DAG(
    schedule="0 0 * * *",
    start_date=datetime.fromisoformat("2025-01-01T00:00:00-04:56"),
    max_active_tasks=1,
    max_active_runs=1,
    catchup=False,
    params={
        "delete_successful": Param(True, title="Delete Successful", description=None, type="boolean"),
        "delete_failed": Param(True, title="Delete Failed", description=None, type="boolean"),
        "mark_failed_as_successful": Param(False, title="Mark Failed As Successful", description=None, type="boolean"),
        "max_dagruns": Param(10, title="Max Dagruns", description=None, type="integer"),
        "days_to_keep": Param(10, title="Days To Keep", description=None, type="integer"),
    },
    dag_id="test_cleanup",
    default_args={},
) as dag:
    cleanup_task = create_cleanup_dag_runs(task_id="cleanup_task", dag=dag)
"""
        )



from datetime import datetime, timedelta

from airflow_pydantic import BaseModel
from pytz import UTC

__all__ = ("DagCleanup",)


class DagCleanup(BaseModel):
    delete_successful: bool = True
    delete_failed: bool = True

    mark_failed_as_successful: bool = False

    max_dagruns: int = 10
    days_to_keep: int = 10

    @property
    def cleanup_dag_runs(self):
        from airflow.models import DagModel, DagRun
        from airflow.utils.session import provide_session
        from airflow.utils.state import State

        @provide_session
        def _cleanup_dag_runs(session=None, **context):
            params = context["params"]

            # Get the configurable parameters
            delete_successful = params.get("delete_successful", DagCleanup.model_fields["delete_successful"].default)
            delete_failed = params.get("delete_failed", DagCleanup.model_fields["delete_failed"].default)
            mark_failed_as_successful = params.get("mark_failed_as_successful", DagCleanup.model_fields["mark_failed_as_successful"].default)
            max_dagruns = params.get("max_dagruns", DagCleanup.model_fields["max_dagruns"].default)
            days_to_keep = params.get("days_to_keep", DagCleanup.model_fields["days_to_keep"].default)

            # Make cutoff_date timezone-aware (UTC)
            utc_now = datetime.utcnow().replace(tzinfo=UTC)
            cutoff_date = utc_now - timedelta(days=days_to_keep)

            # Fetch all DAGs from the DagBag
            dag_ids = [d.dag_id for d in session.query(DagModel.dag_id).distinct(DagModel.dag_id).all()]

            deleted = 0

            for dag_id in dag_ids:
                print(f"Cleaning up DAG: {dag_id}")

                # Query for DAG runs of each DAG
                query = session.query(DagRun).filter(DagRun.dag_id == dag_id)

                if delete_successful is False:
                    query = query.filter(DagRun.state != State.SUCCESS)
                if delete_failed is False:
                    query = query.filter(DagRun.state != State.FAILED)

                dagruns = query.order_by(DagRun.execution_date.asc()).all()
                total_runs = len(dagruns)

                for dr in dagruns:
                    # Compare execution_date (offset-aware) with cutoff_date (now offset-aware)
                    if dr.execution_date < cutoff_date or total_runs > max_dagruns:
                        session.delete(dr)
                        deleted += 1
                        total_runs -= 1  # Adjust count since we deleted one
                    elif mark_failed_as_successful:
                        # Need to iterate through all remaining
                        if dr.state == State.FAILED:
                            # Mark failed runs as successful
                            dr.state = State.SUCCESS
                            session.merge(dr)
                    elif not mark_failed_as_successful:
                        break  # Since they are ordered, no more to delete

            session.commit()
            print(f"Total DAG runs deleted: {deleted}")

        return _cleanup_dag_runs



from airflow_config import load_config


class TestConfig:
    def test_load_config_hydra(self):
        config = load_config(config_name="config")
        assert config
        assert "libraries" in config.extensions
        assert len(config.extensions["libraries"].pip) == 3
        assert len(config.extensions["libraries"].git) == 3



from subprocess import call
from typing import Callable

from airflow_pydantic.airflow import AirflowSkipException

__all__ = ("ping",)


def ping(host, *, local=True) -> Callable:
    if host != "localhost" and host.count(".") == 0 and local:
        host = f"{host}.local"

    def _ping(hostname=host):
        if call(["ping", "-c", "1", f"{hostname}"]) == 0:
            return True
        raise AirflowSkipException

    return _ping



from airflow_pydantic import in_bash

__all__ = ("in_bash", "in_conda", "in_virtualenv")


def in_conda(env: str, command: str, tool: str = "micromamba") -> str:
    return f"{tool} activate {env} && {command}"


def in_virtualenv(env: str, command: str) -> str:
    return f"source {env}/bin/activate && {command}"
